@@ -74,16 +74,34 @@ encodeFolderPath <- function(folderPath=NULL)
         ## URL encoding of folderPath
         folderPath <- URLencode(folderPath)
 
-        ## Formatting
-        folderPath <- gsub("[\\]", "/", folderPath)
-        if(substr(folderPath, nchar(folderPath), nchar(folderPath))!="/")
-            folderPath <- paste(folderPath,"/",sep="")
-        if(substr(folderPath, 1, 1)!="/")
-            folderPath <- paste("/",folderPath,sep="")
+        folderPath <- normalizeSlash(folderPath)
     }
     return (folderPath)
 }
 
+normalizeSlash <- function(folderPath, leading = T, trailing = T) {
+  ## Formatting
+  folderPath <- gsub("[\\]", "/", folderPath)
+  
+  if (trailing) {
+    if(substr(folderPath, nchar(folderPath), nchar(folderPath))!="/")
+      folderPath <- paste(folderPath,"/",sep="")
+  } else {
+    if(substr(folderPath, nchar(folderPath), nchar(folderPath))=="/")
+      folderPath <- substr(folderPath,1, nchar(folderPath)-1)
+  }
+  
+  if (leading) {
+    if(substr(folderPath, 1, 1)!="/")
+      folderPath <- paste("/",folderPath,sep="")
+  } else {
+    if(substr(folderPath, 1, 1)=="/")
+      folderPath <- substr(folderPath,2, nchar(folderPath))
+  }
+  
+  return(folderPath)
+}
+
 ## helper to retrieve and cache the CSRF token
 labkey.getCSRF <- function()
 {
@@ -181,7 +199,7 @@ labkey.get <- function(myurl)
 }
 
 ## Executes an HTTP POST of pbody against the supplied URL, with standard handling for session, api key, status codes and error messages.
-labkey.post <- function(myurl, pbody, encoding=NULL)
+labkey.post <- function(myurl, pbody, encoding=NULL, responseType=NULL, haltOnError=TRUE)
 {
     ## HTTP POST form
     options <- labkey.getRequestOptions(method="POST", encoding=encoding)
@@ -190,28 +208,16 @@ labkey.post <- function(myurl, pbody, encoding=NULL)
         response <- POST(url=myurl, config=options, body=pbody, verbose(data_in=TRUE, info=TRUE, ssl=TRUE))
     else
         response <- POST(url=myurl, config=options, body=pbody)
-    processResponse(response)
+    processResponse(response, responseType = responseType, haltOnError = haltOnError)
 }
 
-processResponse <- function(response, haltOnError=TRUE)
+processResponse <- function(response, haltOnError=TRUE, responseType = NULL)
 {
-    ## Error checking, decode data and return
-    status_code <- response$status_code
-
-    status <- http_status(response)
-    if(status_code==500 | status_code >= 400)
+    if(isRequestError(response))
     {
-        ## pull out the error message if possible
-        error <- content(response, type = "application/json")
-        message = status$message
-        if (!is.null(error$exception))
-        {
-            message <- error$exception
-        }
-        if (haltOnError)
-            stop (paste("HTTP request was unsuccessful. Status code = ", status_code, ", Error message = ", message, sep=""))
+      handleError(response, haltOnError)
     }
-    content(response, "text")
+    content(response, as = "text", type = responseType)
 }
 
 labkey.setDebugMode <- function(debug=FALSE)
@@ -219,6 +225,54 @@ labkey.setDebugMode <- function(debug=FALSE)
     .lkdefaults[["debug"]] = debug;
 }
 
+isRequestError <- function(response, status_code) 
+{
+  status_code <- getStatusCode(response)
+  
+  return(status_code==500 | status_code >= 400)
+}
+
+getStatusCode <- function(response) 
+{
+  ## Error checking, decode data and return
+  status_code <- response$status_code
+  
+  #test for the situations where the header reports 200, but the JSON contains the error:
+  if (sum(grepl('application/json', response$headers[['content-type']])) > 0) {
+    c <- content(response, type = "application/json")
+    if (!is.null(c[['status']])) {
+      status_code <- c$status
+    }
+  }
+
+  return(status_code)  
+}
+
+handleError <- function(response, haltOnError) 
+{
+  status <- http_status(response)
+  message = status$message
+  
+  ## pull out the error message if possible
+  error <- content(response, type = "application/json")
+  if (!is.null(error$exception))
+  {
+    message <- error$exception
+  }
+  if (haltOnError) {
+    status_code <- getStatusCode(response)
+
+    # Note: is this request was writing to a file, the error message JSON will be written to that file, so we delete.  
+    if (inherits(response$content, 'path')) {
+      if (file.exists(response$content)){
+        unlink(response$content)  
+      } 
+    }
+
+    stop (paste("HTTP request was unsuccessful. Status code = ", status_code, ", Error message = ", message, sep=""))
+  }
+}
+
 verboseOutput <- function(title, content)
 {
     if (!is.null(.lkdefaults[["debug"]]) && .lkdefaults[["debug"]] == TRUE) {
